@@ -57,43 +57,19 @@ static int threads_per_cpu = 0;
 static int cpus_in_system = 0;
 static int threads_in_system = 0;
 
-static int test_attr(char *path, char *perms)
-{
-	FILE *fp;
+static int do_info(void);
 
-	fp = fopen(path, perms);
-	if (fp) {
-		fclose(fp);
-		return 1;
-	}
-
-	if (errno == ENOENT) {
-		/* When CPU is offline, attr may not exist. Return 2 so caller
-		 * will not see 'false' and continue to try next CPU. */
-		return 2;
-	}
-
-	return 0;
-}
-
-static int attr_is_readable(char *path)
-{
-	return test_attr(path, "r");
-}
-
-static int attr_is_writeable(char *path)
-{
-	return test_attr(path, "w");
-}
-
-static int test_sysattr(char *attribute, char *perms)
+static int test_sysattr(char *attribute, int perms)
 {
 	char path[SYSFS_PATH_MAX];
 	int i;
 
 	for (i = 0; i < threads_in_system; i++) {
 		sprintf(path, SYSFS_CPUDIR"/%s", i, attribute);
-		if (!test_attr(path, perms))
+		if (access(path, F_OK))
+			continue;
+
+		if (access(path, perms))
 			return 0;
 	}
 
@@ -102,12 +78,12 @@ static int test_sysattr(char *attribute, char *perms)
 
 static int sysattr_is_readable(char *attribute)
 {
-	return test_sysattr(attribute, "r");
+	return test_sysattr(attribute, R_OK);
 }
 
 static int sysattr_is_writeable(char *attribute)
 {
-	return test_sysattr(attribute, "w");
+	return test_sysattr(attribute, W_OK);
 }
 
 static int get_attribute(char *path, const char *fmt, int *value)
@@ -115,14 +91,14 @@ static int get_attribute(char *path, const char *fmt, int *value)
 	FILE *fp;
 	int rc;
 
+	rc = access(path, F_OK);
+	if (rc)
+		return -1;
+
+
 	fp = fopen(path, "r");
-	if (fp == NULL) {
-		if (errno == ENOENT)
-			/* No attribute, cpu probably offline */
-			return 0;
-		else
-			return -1;
-	}
+	if (!fp)
+		return -1;
 
 	rc = fscanf(fp, fmt, value);
 	fclose(fp);
@@ -182,6 +158,9 @@ static int get_system_attribute(char *attribute, const char *fmt, int *value,
 	for (i = 0; i < threads_in_system; i++) {
 		int cpu_attribute;
 
+		if (!cpu_online(i))
+			continue;
+
 		sprintf(path, SYSFS_CPUDIR"/%s", i, attribute);
 		rc = get_attribute(path, fmt, &cpu_attribute);
 		if (rc)
@@ -239,7 +218,7 @@ static int set_dscr(int state)
 	int rc;
 
 	if (dscr_default_exists()) {
-		if (!attr_is_writeable(DSCR_DEFAULT_PATH)) {
+		if (access(DSCR_DEFAULT_PATH, W_OK)) {
 			perror("Cannot set default dscr value");
 			return -2;
 		}
@@ -262,7 +241,7 @@ static int get_dscr(int *value, int *inconsistent)
 	int rc;
 
 	if (dscr_default_exists()) {
-		if (!attr_is_readable(DSCR_DEFAULT_PATH)) {
+		if (access(DSCR_DEFAULT_PATH, R_OK)) {
 			perror("Cannot retrieve default dscr");
 			return -2;
 		}
@@ -313,7 +292,7 @@ static int offline_thread(const char *path)
 
 static int is_subcore_capable(void)
 {
-	return attr_is_readable(SYSFS_SUBCORES) == 1;
+	return access(SYSFS_SUBCORES, F_OK) == 0;
 }
 
 static int num_subcores(void)
@@ -550,6 +529,11 @@ static int do_smt(char *state)
 	return rc;
 }
 
+static inline void do_threads_per_core()
+{
+	printf("Threads per core: %d\n", threads_per_cpu);
+}
+
 static int do_subcores_per_core(char *state)
 {
 	int rc = 0;
@@ -576,13 +560,9 @@ static int do_subcores_per_core(char *state)
 		}
 		printf("Subcores per core: %d\n", subcore_state);
 	} else {
+                /* Kernel decides what values are valid, so no need to
+                 * check here. */
 		subcore_state = strtol(state, NULL, 0);
-		/* Validate option: we only suport 1 or 4. */
-		if (subcore_state != 1 && subcore_state != 4) {
-			fprintf(stderr, "subcores-per-core=%d invalid.\n", subcore_state);
-			return -1;
-		}
-
 		rc = set_attribute(SYSFS_SUBCORES, "%d", subcore_state);
 		if (rc) {
 			fprintf(stderr, "Failed to set subcore option.\n");
@@ -739,7 +719,7 @@ static int do_run_mode(char *run_mode)
 			return -1;
 		}
 
-		*(short *)mode = 1;
+		*(short *)mode = htobe16(1);
 		mode[2] = rmode;
 
 		rc = rtas_set_sysparm(DIAGNOSTICS_RUN_MODE, mode);
@@ -1028,10 +1008,9 @@ static int do_cpu_frequency(void)
 
 #endif
 
-static int do_cores_present(char * state)
+static inline void do_cores_present()
 {
 	printf("Number of cores present = %d\n", cpus_in_system);
-	return 0;
 }
 
 static int set_all_threads_off(int cpu, int smt_state)
@@ -1091,7 +1070,9 @@ static int do_cores_online(char *state)
 
 	smt_state = get_smt_state();
 	if (smt_state == -1) {
-		printf("Bad or inconsistent SMT state\n");
+		printf("Bad or inconsistent SMT state: use ppc64_cpu --smt=on|off to set all\n"
+                       "cores to have the same number of online threads to continue.\n");
+		do_info();
 		return -1;
 	}
 
@@ -1114,7 +1095,7 @@ static int do_cores_online(char *state)
 
 	if (number_to_have > cpus_in_system) {
 		printf("Cannot online more cores than are present.\n");
-		do_cores_present(NULL);
+		do_cores_present();
 		return -1;
 	}
 
@@ -1149,7 +1130,7 @@ static int do_cores_online(char *state)
 	return 0;
 }
 
-static int do_info(char *arg)
+static int do_info(void)
 {
 	int i, j, thread_num;
 	char online;
@@ -1200,6 +1181,7 @@ static void usage(void)
 "                                    # seconds, default is 1 second.\n\n"
 "ppc64_cpu --subcores-per-core       # Get number of subcores per core\n"
 "ppc64_cpu --subcores-per-core=X     # Set subcores per core to X (1 or 4)\n"
+"ppc64_cpu --threads-per-core        # Get threads per core\n"
 "ppc64_cpu --info                    # Display system state information)\n");
 }
 
@@ -1301,13 +1283,15 @@ int main(int argc, char *argv[])
 	else if (!strcmp(action, "frequency"))
 		rc = do_cpu_frequency(sleep_time);
 	else if (!strcmp(action, "cores-present"))
-		rc = do_cores_present(action_arg);
+		do_cores_present();
 	else if (!strcmp(action, "cores-on"))
 		rc = do_cores_online(action_arg);
 	else if (!strcmp(action, "subcores-per-core"))
 		rc = do_subcores_per_core(action_arg);
+	else if (!strcmp(action, "threads-per-core"))
+		do_threads_per_core();
 	else if (!strcmp(action, "info"))
-		rc = do_info(action_arg);
+		rc = do_info();
 	else if (!strcmp(action, "version"))
 		printf("ppc64_cpu: version %s\n", PPC64_CPU_VERSION);
 	else
